<?php

namespace Drupal\fpdi_print\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface as CoreRendererInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\StreamedResponse;
use Symfony\Component\Yaml\Yaml;

/**
 * Controller class for printing Views.
 */
class ViewPrintController extends ControllerBase {

  /**
   * The Entity Type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * The renderer for renderable arrays.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, Request $current_request, AccountInterface $current_user, CoreRendererInterface $renderer) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentRequest = $current_request;
    $this->currentUser = $current_user;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('current_user'),
      $container->get('renderer')
    );
  }

  /**
   * Print an entity to the selected format.
   *
   * @param string $export_type
   *   The export type.
   * @param string $view_name
   *   The view name.
   * @param string $display_id
   *   The view display to render.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response object on error otherwise the Print is sent.
   */
  public function viewPrint($view_name, $display_id, $option_id) {
    // Create the Print engine plugin.
    $config = $this->config('fpdi_print.settings');
    /** @var \Drupal\views\Entity\View $view */
    $view = $this->entityTypeManager->getStorage('view')->load($view_name);
    $executable = $view->getExecutable();
    $executable->setDisplay($display_id);
    [$areaType, $optionId] = explode('-',$option_id);
    $option = $executable->getHandlers($areaType);
    $positions = [];
    if ($args = $this->currentRequest->query->get('view_args')) {
      $executable->setArguments($args);
    }
    $executable->execute();
    // Get 1 the results of the view.
    $view_result = current($executable->result);
    $filePdfTemplate = NULL;
    $rationPixel2mm = 0.27; // 100px = 0.27mm (75dpi)
    if(!empty($option[$optionId])){
      $positionText = $option[$optionId]['position_text'];
      $filePdfTemplate = $option[$optionId]['file_pdf'];
      if(!empty($positionText)){
        $positionText = Yaml::parse($positionText);
        if(!empty($positionText)){
          $style = $executable->getStyle();
          $pageNoDefault = 1;
          foreach ($positionText as &$position){
            $pageNo = !empty($position['page']) ? $position['page'] : $pageNoDefault;
            if(!empty($position['text'])) {
              $value = $style->globalTokenReplace($style->tokenizeValue($position['text'], 0));
              if (strpos($position['text'], '{{') !== FALSE && empty($value) && !empty($view_result)) {
                $fieldName = trim(str_replace(['{{','}}'], '', $position['text']));
                if (!empty($executable->field[$fieldName])) {
                  $value = $executable->field[$fieldName]->advancedRender($view_result)->__toString();
                }
              }
              $position['text'] = $value;
              $positions[$pageNo][] = $position;
            } elseif(!empty($position['image'])) {
              $doc = new \DOMDocument();
              @$doc->loadHTML($position['image']);
              $tags = $doc->getElementsByTagName('img');
              //if there is more than 1 image we move next image 50 point
              $widthDefault = !empty($position['width']) ? $position['width'] : 80;
              if(!empty($tags->length)){
                foreach ($tags as $i=>$tag) {
                  $width = !empty($position['width']) ? $position['width'] : $widthDefault = $tag->getAttribute('width');
                  $height = !empty($position['height']) ? $position['height'] : $tag->getAttribute('height');
                  $positions[$pageNo][] = [
                    'image' => $tag->getAttribute('src'),
                    'x' => $position['x'] + $i*$widthDefault*$rationPixel2mm,
                    'y' => $position['y'],
                    'width' => $width*$rationPixel2mm,
                    'height' => $height*$rationPixel2mm,
                  ];
                }
              }else{
                $positions[$pageNo][] = $position;
              }
            }elseif(!empty($position['html'])){
              $positions[$pageNo][] = $position;
            }
          }
        }
      }
    }

    $pdf = \Drupal::service('fpdi_print.print_builder')->getPDF($positions, $filePdfTemplate);
    $response = new StreamedResponse();
    return $response->setCallback(function () use ($pdf) {
      ob_clean();
      $handle = fopen('php://output', 'w+');
      $pdf->Output();
      fclose($handle);
    });
  }

  /**
   * Validate that the current user has access.
   *
   * We need to validate that the user is allowed to access views display id
   *
   * @param string $view_name
   *   The view name.
   * @param string $display_id
   *   The view display to render.
   *
   * @param string $option_id
   *   The option id.
   * @return \Drupal\Core\Access\AccessResult
   *   The access result object.
   */
  public function checkAccess($view_name, $display_id, $option_id = '') {
    $view = $this->entityTypeManager->getStorage('view')->load($view_name)->getExecutable();
    $account = $this->currentUser();

    // Check the content permission.
    $result = AccessResult::allowedIfHasPermission($account, 'access content');

    // Also check the permissions defined by the view.
    return $result->isAllowed() && $view->access($display_id, $account) ? $result : AccessResult::forbidden();
  }
}
